import rbt_node


class RedBlackTree:
    def __init__(self, root=None):
        self.root = root

    def rotate_counterclockwise(self, a, b):
        # "a" is the parent, and "b" is the right child
        a.right_child = b.left_child

        if b.left_child:
            a.right_child.parent = a

        b.parent = a.parent
        if not b.parent:
            self.root = b
        elif b.parent.left_child == a:
            b.parent.left_child = b
        else:  # "a" was a right child
            b.parent.right_child = b

        b.left_child = a
        a.parent = b

    def rotate_clockwise(self, b, a):
        # "b" is the parent, and "a" is the left child
        b.left_child = a.right_child
        
        if a.right_child:
            b.left_child.parent = b

        a.parent = b.parent
        if not a.parent:
            self.root = a
        elif a.parent.right_child == b:
            a.parent.right_child = a
        else:  # "b" was a left child
            a.parent.left_child = a

        a.right_child = b
        b.parent = a
    def insert(self, value):
        new_node = rbt_node.Node(value, "red")

        if not self.root:
            self.root = new_node
            return
        
        current_node = self.root

        while current_node:

            if value < current_node.value:
                if not current_node.left_child:
                    current_node.left_child = new_node
                    new_node.parent = current_node

                current_node = current_node.left_child

            elif value > current_node.value:
                if not current_node.right_child:
                    current_node.right_child = new_node
                    new_node.parent = current_node

                current_node = current_node.right_child

            else:  # value is already inside tree
                break

        self.fix_insert(new_node)
    def fix_insert(self, current_node):
        # Step 1 was accomplished by the `insert` method
        # which calls this fix_insert method
        while self.has_red_parent(current_node):           # Step 2
            if current_node.parent == self.root:           # Step 3
                self.root.color = "black"
                return
            
            uncle = self.find_uncle(current_node)          # Step 4
            if uncle and uncle.color == "red":             # Step 6
                current_node.parent.color = "black"        # Step 7
                uncle.color = "black"                      # Step 8
                current_node.parent.parent.color = "red"   # Step 9
                current_node = current_node.parent.parent  # Step 10
            else:  # uncle is None or black                  Step 5
                break

        if self.has_red_parent(current_node):
            # The next line calls the straighten_segment method
            # which accomplishes Steps 11, 12, and 13
            current_node = self.straighten_segment(current_node, 
                                                   current_node.parent)
            current_node.parent.color = "black"               # Step 14
            current_node.parent.parent.color = "red"          # Step 15
            self.perform_final_rotation(current_node.parent)  # Step 16

    def has_red_parent(self, node):
         return node.parent and node.parent.color == "red"
    
    def parent_is_root(self, node):
        return node.parent == self.root

    def is_a_left_child(self, node):
        return node == node.parent.left_child
    
    def is_a_right_child(self, node):
        return node == node.parent.right_child

    def find_uncle(self, node):
        if self.is_a_left_child(node.parent):
            return node.parent.parent.right_child
        else:  # parent is a right child
            return node.parent.parent.left_child
        
    def straighten_segment(self, node, parent):
        former_parent = parent
        if self.is_a_left_child(parent) and self.is_a_right_child(node):
            self.rotate_counterclockwise(parent, node)
            return former_parent
        elif self.is_a_right_child(parent) and self.is_a_left_child(node):
            self.rotate_clockwise(parent, node)
            return former_parent
        else:  # no need to straighten the segment
            return node

    def perform_final_rotation(self, node):
        # If the node's parent is a left child of its OWN parent:
        if self.is_a_left_child(node.parent):
            self.rotate_clockwise(node.parent, node)
        else:  # if the parent is instead a right child:
            self.rotate_counterclockwise(node.parent, node)
