import rbt_node
import rbt


def assert_equal(x, y):
    if x == y:
        print(".")
    else:
        print("FAIL")


# Rotate Counterclockwise
print("Rotate counterclockwise")
# a is root
a = rbt_node.Node(5, "black")
b = rbt_node.Node(10, "red")
a.right_child = b
b.parent = a
tree = rbt.RedBlackTree(a)
tree.rotate_counterclockwise(a, b)
assert_equal(b.left_child, a)
assert_equal(b.parent, None)
assert_equal(a.parent, b)
assert_equal(tree.root, b)
assert_equal(a.right_child, None)

# a is left child
root = rbt_node.Node(7, "black")
a = rbt_node.Node(5, "black")
b = rbt_node.Node(10, "red")
root.left_child = a
a.parent = root
a.right_child = b
b.parent = a
tree = rbt.RedBlackTree(root)
tree.rotate_counterclockwise(a, b)
assert_equal(tree.root, root)
assert_equal(b.parent, root)
assert_equal(root.left_child, b)

# a is right child
root = rbt_node.Node(3, "black")
a = rbt_node.Node(5, "black")
b = rbt_node.Node(10, "red")
root.right_child = a
a.parent = root
a.right_child = b
b.parent = a
tree = rbt.RedBlackTree(root)
tree.rotate_counterclockwise(a, b)
assert_equal(root.right_child, b)

# crossover node + other nodes
root = rbt_node.Node(3, "black")
a = rbt_node.Node(5, "black")
b = rbt_node.Node(10, "red")
a_left_child = rbt_node.Node(1, "black")
a.left_child = a_left_child
a_left_child.parent = a
b_right_child = rbt_node.Node(20, "black")
b.right_child = b_right_child
b_right_child.parent = b
crossover_node = rbt_node.Node(7, "black")
b.left_child = crossover_node
crossover_node.parent = b
root.right_child = a
a.parent = root
a.right_child = b
b.parent = a
tree = rbt.RedBlackTree(root)
tree.rotate_counterclockwise(a, b)
assert_equal(a.left_child, a_left_child)
assert_equal(a_left_child.parent, a)
assert_equal(b.right_child, b_right_child)
assert_equal(b_right_child.parent, b)
assert_equal(a.right_child, crossover_node)
assert_equal(crossover_node.parent, a)
# sanity check
assert_equal(b.left_child, a)
assert_equal(a.parent, b)
assert_equal(b.parent, root)
assert_equal(root.right_child, b)

# Rotate Clockwise
print("Rotate clockwise")
# a is root
b = rbt_node.Node(10, "black")
a = rbt_node.Node(5, "red")
b.left_child = a
a.parent = b
tree = rbt.RedBlackTree(b)
tree.rotate_clockwise(b, a)
assert_equal(a.right_child, b)
assert_equal(a.parent, None)
assert_equal(b.parent, a)
assert_equal(tree.root, a)
assert_equal(b.left_child, None)

# b is right child
root = rbt_node.Node(7, "black")
b = rbt_node.Node(10, "black")
a = rbt_node.Node(5, "red")
root.right_child = b
b.parent = root
b.left_child = a
a.parent = b
tree = rbt.RedBlackTree(root)
tree.rotate_clockwise(b, a)
assert_equal(tree.root, root)
assert_equal(a.parent, root)
assert_equal(root.right_child, a)

# b is a left child
root = rbt_node.Node(12, "black")
b = rbt_node.Node(10, "black")
a = rbt_node.Node(5, "red")
root.left_child = b
b.parent = root
b.left_child = a
a.parent = b
tree = rbt.RedBlackTree(root)
tree.rotate_clockwise(b, a)
assert_equal(root.left_child, a)

# crossover node + other nodes
root = rbt_node.Node(12, "black")
b = rbt_node.Node(10, "black")
a = rbt_node.Node(5, "red")
a_left_child = rbt_node.Node(1, "black")
a.left_child = a_left_child
a_left_child.parent = a
b_right_child = rbt_node.Node(11, "black")
b.right_child = b_right_child
b_right_child.parent = b
crossover_node = rbt_node.Node(7, "black")
a.right_child = crossover_node
crossover_node.parent = a
root.left_child = b
b.parent = root
b.left_child = a
a.parent = b
tree = rbt.RedBlackTree(root)
tree.rotate_clockwise(b, a)
assert_equal(a.left_child, a_left_child)
assert_equal(a_left_child.parent, a)
assert_equal(b.right_child, b_right_child)
assert_equal(b_right_child.parent, b)
assert_equal(b.left_child, crossover_node)
assert_equal(crossover_node.parent, b)
# sanity check
assert_equal(a.right_child, b)
assert_equal(b.parent, a)
assert_equal(a.parent, root)
assert_equal(root.left_child, a)

# Insert (without fixup)
# print("Insert (without fixup)")
# tree = rbt.RedBlackTree()
# tree.insert(10)
# assert_equal(tree.root.value, 10)
# assert_equal(tree.root.color, "black")
# tree.insert(5)
# assert_equal(tree.root.value, 10)
# left = tree.root.left_child
# assert_equal(left.value, 5)
# assert_equal(left.color, "red")
# assert_equal(left.parent, tree.root)
# tree.insert(15)
# assert_equal(tree.root.value, 10)
# right = tree.root.right_child
# assert_equal(right.value, 15)
# assert_equal(right.color, "red")
# assert_equal(right.parent, tree.root)
# tree.insert(7)
# assert_equal(tree.root.left_child.right_child.value, 7)
# tree.insert(13)
# assert_equal(tree.root.right_child.left_child.value, 13)

# Insert (with fixup)
print("Insert (with fixup)")
tree = rbt.RedBlackTree()
tree.insert(10)
tree.insert(5)
tree.insert(15)

tree.insert(8)
assert_equal(tree.root.left_child.right_child.value, 8)
assert_equal(tree.root.left_child.right_child.color, "red")
assert_equal(tree.root.left_child.color, "black")
assert_equal(tree.root.right_child.color, "black")
assert_equal(tree.root.color, "red")

tree.insert(3)
tree.insert(1)
assert_equal(tree.root.left_child.color, "red")
assert_equal(tree.root.left_child.left_child.left_child.value, 1)
assert_equal(tree.root.left_child.left_child.left_child.color, "red")
assert_equal(tree.root.left_child.left_child.value, 3)
assert_equal(tree.root.left_child.left_child.color, "black")
assert_equal(tree.root.left_child.right_child.value, 8)
assert_equal(tree.root.left_child.right_child.color, "black")

# double rotation
tree.insert(2)
assert_equal(tree.root.left_child.left_child.left_child.value, 1)
assert_equal(tree.root.left_child.left_child.left_child.color, "red")
assert_equal(tree.root.left_child.left_child.right_child.value, 3)
assert_equal(tree.root.left_child.left_child.right_child.color, "red")
assert_equal(tree.root.left_child.left_child.value, 2)
assert_equal(tree.root.left_child.left_child.color, "black")

tree.insert(20)
tree.insert(13)
tree.insert(25)
tree.insert(23)
assert_equal(tree.root.right_child.color, "red")  # 15
assert_equal(tree.root.right_child.left_child.color, "black")  # 13
assert_equal(tree.root.right_child.right_child.color, "black")  # 20
assert_equal(tree.root.right_child.right_child.right_child.color, "red")  # 25

assert_equal(tree.root.right_child.right_child.value, 23)
assert_equal(tree.root.right_child.right_child.color, "black")
assert_equal(tree.root.right_child.right_child.right_child.value, 25)
assert_equal(tree.root.right_child.right_child.right_child.color, "red")
assert_equal(tree.root.right_child.right_child.left_child.value, 20)
assert_equal(tree.root.right_child.right_child.left_child.color, "red")

# single rotation
print("single rotation due to null uncle")
tree = rbt.RedBlackTree()
tree.insert(10)
tree.insert(15)
tree.insert(5)
tree.insert(20)
tree.insert(25)
assert_equal(tree.root.right_child.value, 20)
assert_equal(tree.root.right_child.color, "black")
assert_equal(tree.root.right_child.right_child.value, 25)
assert_equal(tree.root.right_child.right_child.color, "red")
assert_equal(tree.root.right_child.left_child.value, 15)
assert_equal(tree.root.right_child.left_child.color, "red")
