import treap_node


class Treap:
    def __init__(self, root=None):
        self.root = root

    def rotate_counterclockwise(self, a, b):
        a.right_child = b.left_child

        if b.left_child:
            a.right_child.parent = a

        b.parent = a.parent
        if not b.parent:
            self.root = b
        elif b.parent.left_child == a:
            b.parent.left_child = b
        else:  # Node B is a right child
            b.parent.right_child = b

        b.left_child = a
        a.parent = b

    def rotate_clockwise(self, b, a):
        b.left_child = a.right_child

        if a.right_child:
            b.left_child.parent = b

        a.parent = b.parent
        if not a.parent:
            self.root = a
        elif a.parent.right_child == b:
            a.parent.right_child = a
        else:  # Node A is a left child
            a.parent.left_child = a

        a.right_child = b
        b.parent = a

    def is_a_left_child(self, node):
        return node == node.parent.left_child

    def is_a_right_child(self, node):
        return node == node.parent.right_child

    def insert(self, value, priority=None):
        new_node = treap_node.Node(value, priority)

        if not self.root:
            self.root = new_node
            return

        current_node = self.root

        while current_node:
            if value < current_node.value:
                if not current_node.left_child:
                    current_node.left_child = new_node
                    new_node.parent = current_node

                current_node = current_node.left_child

            elif value > current_node.value:
                if not current_node.right_child:
                    current_node.right_child = new_node
                    new_node.parent = current_node

                current_node = current_node.right_child

            else:  # value is already inside tree
                break

        self.insert_fix(new_node)

    def insert_fix(self, node):
        while node.parent and node.priority < node.parent.priority:
            if self.is_a_left_child(node):
                self.rotate_clockwise(node.parent, node)
            else:
                self.rotate_counterclockwise(node.parent, node)

    def search(self, value):
        if not self.root:
            return None

        current_node = self.root

        while current_node:
            if value < current_node.value:
                current_node = current_node.left_child
            elif value > current_node.value:
                current_node = current_node.right_child
            else:  # value found!
                return current_node

    def delete(self, value):
        node_to_delete = self.search(value)

        if not node_to_delete:
            return False

        if node_to_delete == self.root \
          and not node_to_delete.left_child \
          and not node_to_delete.right_child:
            self.root = None
            return node_to_delete

        while (node_to_delete.left_child or node_to_delete.right_child):
            if (not node_to_delete.right_child) \
              or (node_to_delete.left_child.priority < 
                  node_to_delete.right_child.priority):
                self.rotate_clockwise(node_to_delete, node_to_delete.left_child)
            else:
                # this else clause occurs if either there is only a right child
                # or the right child has smaller priority than left child
                self.rotate_counterclockwise(node_to_delete, 
                                             node_to_delete.right_child)

        if self.is_a_left_child(node_to_delete):
            node_to_delete.parent.left_child = None
        else:
            node_to_delete.parent.right_child = None

        return node_to_delete

    def tree_levels(self, node, array, level=0):
        if not node:
            return

        if level >= len(array):
            array.insert(level, [])

        # array[level].append({ node.value: round(node.priority, 4)})
        array[level].append(node.value)
        self.tree_levels(node.left_child, array, level + 1)
        self.tree_levels(node.right_child, array, level + 1)
        return array
