import os


class BTree:
    def __init__(self, root=None):
        self.root = root
        self.max_node_size = 4

    def search(self, search_value, node=None):
        node_file = node or self.root
        node_data = self.read_node_file(node_file)
        values = node_data.get('values')
        children = node_data.get('children')

        index = 0
        while index < len(values):      
            current_value = values[index]
            if current_value == search_value:
                return [True, node_file, index]
            if search_value < current_value:
                if children:
                    child_to_follow = children[index]
                    break
                else:
                    return [False, node_file, index]
            index += 1

        # if search_value is greater than all values:
        if search_value > current_value:
            if children:
                child_to_follow = children[len(children) - 1]
            else:
                return [False, node_file, index]

        return self.search(search_value, child_to_follow)

    def read_node_file(self, node_file):
        with open(node_file, 'r') as reader:
            parent = reader.readline().rstrip('\n')
            values = reader.readline().rstrip(',\n').split(',')
            values = list(map(lambda x: int(x), values))
            children = reader.readline().rstrip(',\n').split(',')
            if children[0] == '':
                children = None

        return {'parent': parent, 'values': values, 'children': children} 
    
    def insert(self, value):
        # if tree is empty:
        if not self.root:
            self.create_root(value)
            return
        
        search_result = self.search(value)
        
        # if value is already in tree:
        if search_result[0]:
            return
        
        node_file = search_result[1]
        self.insert_into_node(node_file, value)

    def insert_into_node(self, node_file, value):
        node_data = self.read_node_file(node_file)
        values = node_data.get('values')
        children = node_data.get('children')
        parent = node_data.get('parent')
         
        values.append(value)
        values.sort()

        if len(values) > self.max_node_size:
            self.split_node(node_file, parent, values, children)
        else:
            self.write_to_node_file(node_file, parent, values, children)
    
    def split_node(self, node_file, parent, values, children=None):
        os.remove(node_file)

        median_index = self.max_node_size // 2
        left_node_filename = str(values[0]) + '.csv'
        right_node_filename = str(values[median_index + 1]) + '.csv'

        if parent == 'None':
            parent_node_filename = str(values[median_index]) + '.csv'
        else:
            parent_node_filename = parent

        # Split the current node by creating two new nodes
        # (a left node and right node):
        if children:
            left_children = children[:median_index + 1]
            right_children = children[median_index + 1:]
        else:
            left_children = None
            right_children = None

        self.write_to_node_file(left_node_filename, parent_node_filename,
                                values[:median_index], left_children)
        self.write_to_node_file(right_node_filename, parent_node_filename,
                                values[median_index + 1:], right_children)

        if parent == 'None':
            new_parent = self.write_to_node_file(parent_node_filename,
              'None', [values[median_index]], 
              [left_node_filename, right_node_filename])
            self.root = new_parent
        else:  # if current node has a parent:
            # We will soon split the current node into two new nodes, so we 
            # have to add these nodes to the list of the parent's children:
            parent_data = self.read_node_file(parent_node_filename)
            index_of_node_file = parent_data.get('children').index(node_file)

            updated_children = parent_data.get('children')[:index_of_node_file] + \
              [left_node_filename, right_node_filename] + \
              parent_data.get('children')[index_of_node_file + 1:]
            self.write_to_node_file(parent_node_filename,
              parent_data.get('parent'),
              parent_data.get('values'),
              updated_children)

            # Insert the center value into the parent node:
            self.insert_into_node(parent_node_filename, values[median_index])
            
        # Update the left and right nodes' children to reflect their new parents:
        if left_children:
            for child in left_children:
                child_data = self.read_node_file(child)
                self.write_to_node_file(child, left_node_filename,
                  child_data.get('values'), child_data.get('children'))

        if right_children:
            for child in right_children:
                child_data = self.read_node_file(child)
                self.write_to_node_file(child, right_node_filename,
                  child_data.get('values'), child_data.get('children'))

    def write_to_node_file(self, node_file, parent, values, children=None):
        values_string = ''
        for value in values:
            values_string += str(value) + ','

        if children:
            children_string = ''
            for child in children:
                children_string += str(child) + ','

        with open(node_file, 'w') as writer:
            writer.write(parent + '\n')
            writer.write(values_string)
            if children:    
                writer.write('\n' + children_string)          

        return node_file
    
    def create_root(self, value):
        filename = 'root.csv'
        with open(filename, 'w') as writer:
            writer.write('None\n')
            writer.write(str(value) + ',')
        self.root = filename
