import os
from datetime import datetime


class BTree:
    def __init__(self, root=None):
        self.root = root

    def search(self, search_value, node_to_follow=None):
        node_file = node_to_follow or self.root
        node_data = self.read_node_file(node_file)
        values = node_data.get('values')
        children = node_data.get('children')

        index = 0
        while index < len(values):      
            current_value = int(values[index])
            if current_value == search_value:
                return [search_value, node_file, index]
            if search_value < current_value:
                if not children:
                    return [None, node_file, index]
                else:
                    node_to_follow = children[index]
                    break
            index += 1

        # if the node is full and search_value is greater than all values:
        if search_value > current_value:
            if children:
                node_to_follow = children[len(children) - 1]
            else:
                return [None, node_file, index]

        return self.search(search_value, node_to_follow)

    def insert(self, value):
        # if tree is empty
        if not self.root:
            filename = 'root.txt'
            with open(filename, 'w') as writer:
                writer.write('None\n')
                writer.write(str(value) + ',')
            self.root = filename
            return
        
        search_result = self.search(value)
        
        # if value is already in tree
        if search_result[0]:
            return
        
        node_file = search_result[1]
        self.insert_into_node(node_file, value)

    def insert_into_node(self, node_file, value):
        node_data = self.read_node_file(node_file)
        values = node_data.get('values')
        children = node_data.get('children')
        parent = node_data.get('parent')
         
        values.append(value)
        values.sort()

        if len(values) > 4:
            # split!
            # if parent == 'None':
            new_parent = self.split(node_file, parent, values, children)
            self.root = new_parent
        else:
            self.write_to_node_file(node_file, parent, values, children)
        
        #PLAN: return either current node file or array of new split nodes

    def read_node_file(self, node_file):
        with open(node_file, 'r') as reader:
            parent = reader.readline().rstrip('\n')
            values = reader.readline().rstrip(',\n').split(',')
            values = map(lambda x: int(x), values)
            children = reader.readline().rstrip(',\n').split(',')
            if children[0] == '':
                children = None
        
        return {'parent': parent, 'values': values, 'children': children} 

    # Used!
    def write_to_node_file(self, node_file, parent, values, children=None):
        # if node_file:
        #     x = 1
        #     # with open(filename, 'r') as reader:
        #     #     reader.readline()
        #     #     parent = reader.readline()
        # else:
        #     node_file = str(values[0]) + '.txt'
        #     parent = 'None'

        values_string = ''
        for value in values:
            values_string += str(value) + ','

        if children:
            children_string = ''
            for child in children:
                children_string += str(child) + ','

        # Write back to file
        with open(node_file, 'w') as writer:
            writer.write(parent + '\n')
            writer.write(values_string)
            if children:    
                writer.write('\n' + children_string)          

        return node_file
    
    def split(self, node_file, parent, values, children=None):
        # os.remove(node_file)
        print
        print(node_file)
        # print('split into:')
        
        left_node_filename = str(values[0]) + '.txt'
        right_node_filename = str(values[3]) + '.txt'
        # print(left_node_filename)
        print(right_node_filename)
        print('becomes:')

        if parent == 'None':
            parent_node_filename = str(values[2]) + '.txt'
            self.write_to_node_file(parent_node_filename, 'None', [values[2]], [left_node_filename, right_node_filename])
        else:  # if current node has a parent:
            parent_node_filename = parent
            parent_data = self.read_node_file(parent_node_filename)
            index_of_node_file = parent_data.get('children').index(node_file)

            # We will soon split the current node into two new nodes, so we 
            # have to add these nodes to the list of the parent's children:
            updated_children = parent_data.get('children')[:index_of_node_file] + [left_node_filename, right_node_filename] + parent_data.get('children')[index_of_node_file + 1:]
            self.write_to_node_file(parent_node_filename, parent_data.get('parent'), parent_data.get('values'), updated_children)

            # Insert the center value into the parent node:
            self.insert_into_node(parent_node_filename, values[2])
            
        # Split the current node by creating two new nodes (a left node and right node):
        if children:
            left_children = children[:3]
            right_children = children[3:]
        else:
            left_children = None
            right_children = None

        self.write_to_node_file(left_node_filename, parent_node_filename, values[:2], left_children)
        self.write_to_node_file(right_node_filename, parent_node_filename, values[3:], right_children)
        print(right_node_filename)

        # Update the left and right nodes' children to reflect their new parents:
        if left_children:
            for child in left_children:
                child_data = self.read_node_file(child)
                self.write_to_node_file(child, left_node_filename, child_data.get('values'), child_data.get('children'))

        if right_children:
            for child in right_children:
                child_data = self.read_node_file(child)
                self.write_to_node_file(child, right_node_filename, child_data.get('values'), child_data.get('children'))

        # base case
        # return [left_node_filename, right_node_filename]
        return parent_node_filename
    
    # def read_internal_node(self, node_file):
    #     with open(node_file, 'r') as reader:
    #         parent = reader.readline().rstrip('\n')
    #         values = reader.readline().rstrip('\n').split(',')
    #         children = reader.readline().rstrip('\n').split(',')

    #     return [values, children]

    # def split_and_insert_into_parent(self, data, parent_node, node_file):
    #     left_node = self.write_to_node_file(data[:2])
    #     right_node = self.write_to_node_file(data[3:])
    #     self.assign_parent(left_node, parent_node)
    #     self.assign_parent(right_node, parent_node)
    #     parent_data = self.read_internal_node(parent_node)
    #     values = parent_data[0]
    #     values.append(data[2])
    #     children = parent_data[1]
    #     children.remove(node_file)
    #     values.remove('')
    #     children.append(left_node)
    #     children.append(right_node)
    #     values.sort(key=lambda datum: int(datum))
    #     children.sort(key=lambda datum: int(datum.strip('.txt')))
    #     self.insert_into_internal_node(parent_node, values, children)
    #     os.remove(node_file)


    # def insert_into_internal_node(self, node_file, values, children):
    #     string_1 = ''
    #     string_2 = ''

    #     for i in range(4):
    #         if i < len(children):
    #             string_2 += str(children[i]) + ','
    #         if i < len(values):
    #             string_1 += str(values[i]) + ','

    #     with open(node_file, 'w') as writer:
    #          writer.write('None\n')
    #          writer.write(string_1 + '\n' + string_2) 

    # def create_internal_node(self, data, children, filename=None):
    #     if not filename:
    #         filename = str(data) + '.txt'

    #     string_1 = str(data)
    #     string_2 = str(children[0]) + ',' + str(children[1]) + ','

    #     with open(filename, 'w') as writer:
    #         writer.write(string_1 + '\n' + string_2) 
        
    #     return filename

    # # Garbage, I think
    # def read_node_file(self, node_file):
    #     with open(node_file, 'r') as reader:
    #         parent_node = reader.readline().rstrip('\n')
    #         r = reader.readline().rstrip('\n').split(',')
    #         data = [j for i, j in enumerate(r) if j!='']
    #         data = map(lambda x: int(x), data) 
    #         return [data, parent_node]  

    # def assign_parent(self, node, parent_node):
    #     with open(node, 'a') as writer:
    #         writer.write('\n' + parent_node)  



 # str(time.time()).replace('.', '') + '.txt'

# b = BTree()
# b.insert(5)
# b.insert(7)
# b.insert(3)
# b.insert(4)
# b.insert(2)
# print(b.search(5))

# TODO: need to change node structure so that parent goes on top, values in middle, and children on bottom

