# Global base variables:
base = 26
prime = 613


def find_needle(haystack, needle):
    needle_hash_code = initial_hash(needle)
    window_hash_code = initial_hash(haystack[0:(len(needle))])

    if needle_hash_code == window_hash_code:
        return 0

    # Precompute the "drop place remainder":
    drop_place_remainder = 1
    for i in range(len(needle) - 1):
        drop_place_remainder = (drop_place_remainder * base) % prime

    for i in range(1, len(haystack) - len(needle) + 1):
        drop_character = haystack[i - 1]
        new_character = haystack[i - 1 + len(needle)]
        window_hash_code = rolling_hash(window_hash_code, len(needle), drop_character, new_character, drop_place_remainder)

        if needle_hash_code == window_hash_code:
            # Las Vegas sanity check:
            if needle == haystack[i:(i + len(needle))]:
                return i

    return None


def initial_hash(string):
    result = character_hash_code(string[0]) % prime

    for i in range(1, len(string)):
        result = (result * base + character_hash_code(string[i])) % prime

    return result


def rolling_hash(hash_code, window_length, drop_character, new_character, drop_place_remainder):
    result = ((hash_code + prime - character_hash_code(drop_character) * drop_place_remainder) * base + character_hash_code(new_character)) % prime

    return result


def character_hash_code(char):
    return ord(char) - 97
