import bit_vector
import division_hasher
import math

class BloomFilter:
    def __init__(self, n, m):
        self.m = m
        self.k = int(m * math.log(2) / n)
        self.f = (1 - math.e**((-self.k * n) / m))**self.k

        self.hash_functions = []
        self.hash_function_primes = {}

        for _ in range(self.k):
            hasher = division_hasher.DivisionHasher(self.m)
            while hasher.prime in self.hash_function_primes:
                hasher = division_hasher.DivisionHasher(self.m)

            self.hash_function_primes[hasher.prime] = True
            self.hash_functions.append(hasher)

        self.bv = bit_vector.BitVector(self.m)
    def insert(self, value):
        for hash_function in self.hash_functions:
            hashcode = hash_function.hash(value)
            self.bv.set_bit(hashcode)

    def read(self, value):
        for hash_function in self.hash_functions:
            hashcode = hash_function.hash(value)
            if not self.bv.read_bit(hashcode):
                return False

        return True
